@@ -1,5 +1,6 @@
-/*
- * Copyright (C) 2008 - 2011 Vivien Malerba
+/* gda-data-comparator.c
+ *
+ * Copyright (C) 2008 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -97,8 +98,7 @@ gda_data_comparator_get_type (void)
 			NULL,
 			sizeof (GdaDataComparator),
 			0,
-			(GInstanceInitFunc) gda_data_comparator_init,
-			0
+			(GInstanceInitFunc) gda_data_comparator_init
 		};
 
 		g_static_mutex_lock (&registering);
@@ -110,10 +110,10 @@ gda_data_comparator_get_type (void)
 }
 
 static gboolean
-diff_computed_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
+diff_computed_accumulator (GSignalInvocationHint *ihint,
 			   GValue *return_accu,
 			   const GValue *handler_return,
-			   G_GNUC_UNUSED gpointer data)
+			   gpointer data)
 {
         gboolean thisvalue;
 
@@ -124,7 +124,7 @@ diff_computed_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
 }
 
 static gboolean
-m_diff_computed (G_GNUC_UNUSED GdaDataComparator *comparator, G_GNUC_UNUSED GdaDiff *diff)
+m_diff_computed (GdaDataComparator *comparator, GdaDiff *diff)
 {
         return FALSE; /* default is to allow differences computing to proceed (understand it as: FALSE => don't stop) */
 }
@@ -201,7 +201,7 @@ static void
 clean_diff (GdaDataComparator *comparator)
 {
 	if (comparator->priv->diffs) {
-		gsize i;
+		gint i;
 		for (i = 0; i < comparator->priv->diffs->len; i++) {
 			GdaDiff *diff = g_array_index (comparator->priv->diffs, GdaDiff *, i);
 			gda_diff_free (diff);
@@ -294,9 +294,6 @@ gda_data_comparator_set_property (GObject *object,
 			if (model)
 				g_object_ref (model);
 			break;
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-			break;
 		}
 	}
 }
@@ -318,9 +315,6 @@ gda_data_comparator_get_property (GObject *object,
 		case PROP_NEW_MODEL:
 			g_value_set_object (value, comparator->priv->new_model);
 			break;
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-			break;
 		}
 	}
 }
@@ -351,8 +345,8 @@ gda_data_comparator_set_key_columns (GdaDataComparator *comp, const gint *col_nu
 	comp->priv->key_columns = NULL;
 	if (nb_cols > 0) {
 		comp->priv->nb_key_columns = nb_cols;
-		comp->priv->key_columns = g_new (gint, nb_cols);
-		memcpy (comp->priv->key_columns, col_numbers, sizeof (gint) * nb_cols); /* Flawfinder: ignore */
+		comp->priv->key_columns = g_new0 (gint, nb_cols);
+		memcpy (comp->priv->key_columns, col_numbers, sizeof (gint) * nb_cols);
 	}
 }
 
@@ -421,7 +415,7 @@ find_row_in_model (GdaDataComparator *comp, gint row, gboolean *out_has_changed,
  * @comp: a #GdaDataComparator object
  * @error: a place to store errors, or %NULL
  *
- * Actually computes the differences between the data models for which @comp is defined. 
+ * Actually computes the differences bewteen the data models for which @comp is defined. 
  *
  * For each difference computed, stored in a #GdaDiff structure, the "diff-computed" signal is emitted.
  * If one connects to this signal and returns FALSE in the signal handler, then computing differences will be
@@ -484,21 +478,11 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 
 	/* actual differences computations : rows to insert / update */
 	onrows = gda_data_model_get_n_rows (comp->priv->old_model);
-	if (onrows < 0) {
-		g_set_error (error, GDA_DATA_COMPARATOR_ERROR, GDA_DATA_COMPARATOR_MODEL_ACCESS_ERROR,
-			     "%s", _("Can't get the number of rows of data model to compare from"));
-		return FALSE;
-	}
+	g_assert (onrows >= 0);
 	nnrows = gda_data_model_get_n_rows (comp->priv->new_model);
-	if (nnrows < 0) {
-		g_set_error (error, GDA_DATA_COMPARATOR_ERROR, GDA_DATA_COMPARATOR_MODEL_ACCESS_ERROR,
-			     "%s", _("Can't get the number of rows of data model to compare to"));
-		return FALSE;
-	}
-	if (onrows > 0) {
-		rows_to_del = g_new (gboolean, onrows);
-		memset (rows_to_del, TRUE, sizeof (gboolean) * onrows);
-	}
+	g_assert (nnrows >= 0);
+	rows_to_del = g_new (gboolean, onrows);
+	memset (rows_to_del, TRUE, sizeof (gboolean) * onrows);
 	for (i = 0; i < nnrows; i++) {
 		gint erow = -1;
 		gboolean has_changed = FALSE;
@@ -541,7 +525,6 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 			gint j;
 			diff = g_new0 (GdaDiff, 1);
 			diff->type = GDA_DIFF_MODIFY_ROW;
-			rows_to_del [erow] = FALSE;
 			diff->old_row = erow;
 			diff->new_row = i;
 			diff->values = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free,
@@ -568,9 +551,7 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 						     gda_value_copy (cvalue));
 			}
 		}
-		else
-			rows_to_del [erow] = FALSE; /* row has not been changed */
-
+		rows_to_del [i] = FALSE;
 		if (diff) {
 			g_array_append_val (comp->priv->diffs, diff);
 			g_signal_emit (comp, gda_data_comparator_signals [DIFF_COMPUTED], 0, diff, &stop);
@@ -591,7 +572,7 @@ gda_data_comparator_compute_diff (GdaDataComparator *comp, GError **error)
 		if (rows_to_del [i]) {
 			gint j;
 			diff = g_new0 (GdaDiff, 1);
-			diff->type = GDA_DIFF_REMOVE_ROW;
+			diff->type = GDA_DIFF_ADD_ROW;
 			diff->old_row = i;
 			diff->new_row = -1;
 			diff->values = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free,
