@@ -52,19 +52,17 @@ static void gda_set_finalize (GObject *object);
 static void set_remove_node (GdaSet *set, GdaSetNode *node);
 static void set_remove_source (GdaSet *set, GdaSetSource *source);
 
-
 static void changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
 static GError *validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *dataset);
 static void source_changed_holder_cb (GdaHolder *holder, GdaSet *dataset);
 static void att_holder_changed_cb (GdaHolder *holder, const gchar *att_name, const GValue *att_value, GdaSet *dataset);
-static void holder_notify_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *dataset);
-
 
 static void compute_public_data (GdaSet *set);
 static gboolean gda_set_real_add_holder (GdaSet *set, GdaHolder *holder);
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
+extern GdaAttributesManager *gda_holder_attributes_manager;
 
 /* properties */
 enum
@@ -84,12 +82,10 @@ enum
 	HOLDER_ATTR_CHANGED,
 	VALIDATE_HOLDER_CHANGE,
 	VALIDATE_SET,
-	HOLDER_TYPE_SET,
-	SOURCE_MODEL_CHANGED,
 	LAST_SIGNAL
 };
 
-static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0 };
+static gint gda_set_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
 
 
 /* private structure */
@@ -99,8 +95,6 @@ struct _GdaSetPrivate
 	gchar           *name;
 	gchar           *descr;
 	GHashTable      *holders_hash; /* key = GdaHoler ID, value = GdaHolder */
-	GArray          *holders_array;
-	gboolean         read_only;
 };
 
 static void 
@@ -193,8 +187,7 @@ gda_set_get_type (void)
 			NULL,
 			sizeof (GdaSet),
 			0,
-			(GInstanceInitFunc) gda_set_init,
-			0
+			(GInstanceInitFunc) gda_set_init
 		};
 		
 		g_static_mutex_lock (&registering);
@@ -207,10 +200,10 @@ gda_set_get_type (void)
 }
 
 static gboolean
-validate_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
+validate_accumulator (GSignalInvocationHint *ihint,
 		      GValue *return_accu,
 		      const GValue *handler_return,
-		      G_GNUC_UNUSED gpointer data)
+		      gpointer data)
 {
 	GError *error;
 
@@ -221,14 +214,13 @@ validate_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
 }
 
 static GError *
-m_validate_holder_change (G_GNUC_UNUSED GdaSet *set, G_GNUC_UNUSED GdaHolder *holder,
-			  G_GNUC_UNUSED const GValue *new_value)
+m_validate_holder_change (GdaSet *set, GdaHolder *holder, const GValue *new_value)
 {
 	return NULL;
 }
 
 static GError *
-m_validate_set (G_GNUC_UNUSED GdaSet *set)
+m_validate_set (GdaSet *set)
 {
 	return NULL;
 }
@@ -317,51 +309,11 @@ gda_set_class_init (GdaSetClass *class)
 			      NULL, NULL,
 			      _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
-	/**
-	 * GdaSet::holder-type-set
-	 * @set: the #GdaSet
-	 * @holder: the #GdaHolder for which the #GType has been set
-	 *
-	 * Gets emitted when @holder in @set has its type finally set, in case
-	 * it was #GDA_TYPE_NULL
-	 *
-	 * Since: 4.2
-	 */
-	gda_set_signals[HOLDER_TYPE_SET] =
-		g_signal_new ("holder-type-set",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (GdaSetClass, holder_type_set),
-			      NULL, NULL,
-			      _gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
-			      GDA_TYPE_HOLDER);
-
-	/**
-	 * GdaSet::source-model-changed
-	 * @set: the #GdaSet
-	 * @source: the #GdaSetSource for which the @data_model attribute has changed
-	 *
-	 * Gets emitted when the data model in @source has changed
-	 *
-	 * Since: 4.2
-	 */
-	gda_set_signals[SOURCE_MODEL_CHANGED] =
-		g_signal_new ("source-model-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (GdaSetClass, source_model_changed),
-			      NULL, NULL,
-			      _gda_marshal_VOID__POINTER, G_TYPE_NONE, 1,
-			      G_TYPE_POINTER);
-
-
 	class->holder_changed = NULL;
 	class->validate_holder_change = m_validate_holder_change;
 	class->validate_set = m_validate_set;
 	class->holder_attr_changed = NULL;
 	class->public_data_changed = NULL;
-	class->holder_type_set = NULL;
-	class->source_model_changed = NULL;
 
 	/* Properties */
 	object_class->set_property = gda_set_set_property;
@@ -393,8 +345,6 @@ gda_set_init (GdaSet *set)
 	set->sources_list = NULL;
 	set->groups_list = NULL;
 	set->priv->holders_hash = g_hash_table_new (g_str_hash, g_str_equal);
-	set->priv->holders_array = NULL;
-	set->priv->read_only = FALSE;
 }
 
 
@@ -413,46 +363,18 @@ gda_set_new (GSList *holders)
 {
 	GObject *obj;
 
-	obj = g_object_new (GDA_TYPE_SET, NULL);
-	for (; holders; holders = holders->next) 
-		gda_set_real_add_holder ((GdaSet*) obj, GDA_HOLDER (holders->data));
-	compute_public_data ((GdaSet*) obj);
-
-	return (GdaSet*) obj;
-}
-
-/**
- * gda_set_new_read_only:
- * @holders: (element-type GdaHolder) (transfer:none): a list of #GdaHolder objects
- *
- * Creates a new #GdaSet like gda_set_new(), but does not allow modifications to any of the #GdaHolder
- * object in @holders. This function is used for Libgda's database providers' implementation.
- *
- * Returns: a new #GdaSet object
- *
- * Since: 4.2
- */
-GdaSet *
-gda_set_new_read_only (GSList *holders)
-{
-	GObject *obj;
-
-	obj = g_object_new (GDA_TYPE_SET, NULL);
-	((GdaSet*) obj)->priv->read_only = TRUE;
-	for (; holders; holders = holders->next) 
-		gda_set_real_add_holder ((GdaSet*) obj, GDA_HOLDER (holders->data));
-	compute_public_data ((GdaSet*) obj);
-
-	return (GdaSet*) obj;
+	obj = g_object_new (GDA_TYPE_SET, "holders", holders, NULL);
+	
+	return GDA_SET (obj);
 }
 
 /**
  * gda_set_copy:
  * @set: a #GdaSet object
  *
- * Creates a new #GdaSet object, copy of @set
+ * Creates a new #GdaSet object, opy of @set
  *
- * Returns: (transfer full): a new #GdaSet object
+ * Returns: a new #GdaSet object
  */
 GdaSet *
 gda_set_copy (GdaSet *set)
@@ -473,7 +395,7 @@ gda_set_copy (GdaSet *set)
 }
 
 /**
- * gda_set_new_inline: (skip)
+ * gda_set_new_inline:
  * @nb: the number of value holders which will be contained in the new #GdaSet
  * @...: a serie of a (const gchar*) id, (GType) type, and value
  *
@@ -481,11 +403,11 @@ gda_set_copy (GdaSet *set)
  * For each triplet (id, Glib type and value), 
  * the value must be of the correct type (gchar * if type is G_STRING, ...)
  *
- * Note that this function is a utility function and that only a limited set of types are supported. Trying
+ * Note that this function is a utility function and that anly a limited set of types are supported. Trying
  * to use an unsupported type will result in a warning, and the returned value holder holding a safe default
  * value.
  *
- * Returns: (transfer full): a new #GdaSet object
+ * Returns: a new #GdaSet object
  */ 
 GdaSet *
 gda_set_new_inline (gint nb, ...)
@@ -548,10 +470,6 @@ gda_set_new_inline (gint nb, ...)
 			g_value_set_ulong (value, va_arg (ap, gulong));
 		else if (type == G_TYPE_GTYPE)
 			g_value_set_gtype (value, va_arg(ap, GType));
-		else if (type == GDA_TYPE_TIMESTAMP)
-			gda_value_set_timestamp (value, va_arg(ap, GdaTimestamp *));
-		else if (type == GDA_TYPE_TIME)
-			gda_value_set_time (value, va_arg(ap, GdaTime *));
 		else {
 			g_warning (_("%s() does not handle values of type '%s'."),
 				   __FUNCTION__, g_type_name (type));
@@ -584,15 +502,15 @@ gda_set_new_inline (gint nb, ...)
 }
 
 /**
- * gda_set_set_holder_value: (skip)
+ * gda_set_set_holder_value:
  * @set: a #GdaSet object
- * @error: (allow-none): a place to store errors, or %NULL
+ * @error: a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
  * @...: value, of the correct type, depending on the requested holder's type (not NULL)
  *
  * Set the value of the #GdaHolder which ID is @holder_id to a specified value
  *
- * Returns: %TRUE if no error occurred and the value was set correctly
+ * Returns: TRUE if no error occurred and the value was set correctly
  */
 gboolean
 gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, ...)
@@ -671,7 +589,7 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
  *
  * Get the value of the #GdaHolder which ID is @holder_id
  *
- * Returns: (transfer none): the requested GValue, or %NULL (see gda_holder_get_value())
+ * Returns: the requested GValue, or %NULL (see gda_holder_get_value())
  */
 const GValue *
 gda_set_get_holder_value (GdaSet *set, const gchar *holder_id)
@@ -714,12 +632,12 @@ xml_validity_error_func (void *ctx, const char *msg, ...)
 /**
  * gda_set_new_from_spec_string:
  * @xml_spec: a string
- * @error: (allow-none): a place to store the error, or %NULL
+ * @error: a place to store the error, or %NULL
  *
  * Creates a new #GdaSet object from the @xml_spec
  * specifications
  *
- * Returns: (transfer full): a new object, or %NULL if an error occurred
+ * Returns: a new object, or %NULL if an error occurred
  */
 GdaSet *
 gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
@@ -806,12 +724,12 @@ gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
 /**
  * gda_set_new_from_spec_node:
  * @xml_spec: a #xmlNodePtr for a &lt;holders&gt; tag
- * @error: (allow-none): a place to store the error, or %NULL
+ * @error: a place to store the error, or %NULL
  *
  * Creates a new #GdaSet object from the @xml_spec
  * specifications
  *
- * Returns: (transfer full): a new object, or %NULL if an error occurred
+ * Returns: a new object, or %NULL if an error occurred
  */
 GdaSet *
 gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
@@ -855,9 +773,9 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 				}
 				else  {
 					sources = g_slist_prepend (sources, model);
-					str = (gchar*)xmlGetProp(cur, (xmlChar*) "name");
+					str = (gchar*)xmlGetProp(cur, (xmlChar*)"name");
 					if (str) 
-						g_object_set_data_full (G_OBJECT (model), "name", str, xmlFree);
+						g_object_set_data_full (G_OBJECT (model), "name", str, g_free);
 				}
 			}
 		}
@@ -979,18 +897,14 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 	g_return_if_fail (set->priv);
 	g_return_if_fail (g_slist_find (set->holders, holder));
 
+	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+					      G_CALLBACK (changed_holder_cb), set);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 					      G_CALLBACK (validate_change_holder_cb), set);
-	if (! set->priv->read_only) {
-		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-						      G_CALLBACK (changed_holder_cb), set);
-		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-						      G_CALLBACK (source_changed_holder_cb), set);
-		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
-						      G_CALLBACK (att_holder_changed_cb), set);
-	}
-	g_signal_handlers_disconnect_by_func (holder,
-					      G_CALLBACK (holder_notify_cb), set);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+					      G_CALLBACK (source_changed_holder_cb), set);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
+					      G_CALLBACK (att_holder_changed_cb), set);
 
 	/* now destroy the GdaSetNode and the GdaSetSource if necessary */
 	node = gda_set_get_node (set, holder);
@@ -1008,15 +922,11 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 
 	set->holders = g_slist_remove (set->holders, holder);
 	g_hash_table_remove (set->priv->holders_hash, gda_holder_get_id (holder));
-	if (set->priv->holders_array) {
-		g_array_free (set->priv->holders_array, TRUE);
-		set->priv->holders_array = NULL;
-	}
 	g_object_unref (G_OBJECT (holder));
 }
 
 static void
-source_changed_holder_cb (G_GNUC_UNUSED GdaHolder *holder, GdaSet *set)
+source_changed_holder_cb (GdaHolder *holder, GdaSet *set)
 {
 	compute_public_data (set);
 }
@@ -1038,17 +948,13 @@ validate_change_holder_cb (GdaHolder *holder, const GValue *value, GdaSet *set)
 {
 	/* signal the holder validate-change */
 	GError *error = NULL;
-	if (set->priv->read_only)
-		g_set_error (&error, GDA_SET_ERROR, GDA_SET_READ_ONLY_ERROR, _("Data set does not allow modifications"));
-	else {
 #ifdef GDA_DEBUG_signal
-		g_print (">> 'VALIDATE_HOLDER_CHANGE' from %s\n", __FUNCTION__);
+	g_print (">> 'VALIDATE_HOLDER_CHANGE' from %s\n", __FUNCTION__);
 #endif
-		g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_HOLDER_CHANGE], 0, holder, value, &error);
+	g_signal_emit (G_OBJECT (set), gda_set_signals[VALIDATE_HOLDER_CHANGE], 0, holder, value, &error);
 #ifdef GDA_DEBUG_signal
-		g_print ("<< 'VALIDATE_HOLDER_CHANGED' from %s\n", __FUNCTION__);
+	g_print ("<< 'VALIDATE_HOLDER_CHANGED' from %s\n", __FUNCTION__);
 #endif
-	}
 	return error;
 }
 
@@ -1066,7 +972,7 @@ changed_holder_cb (GdaHolder *holder, GdaSet *set)
 }
 
 static void
-group_free (GdaSetGroup *group, G_GNUC_UNUSED gpointer data)
+group_free (GdaSetGroup *group, gpointer data)
 {
 	g_slist_free (group->nodes);
 	g_free (group);
@@ -1085,17 +991,15 @@ gda_set_dispose (GObject *object)
 	/* free the holders list */
 	if (set->holders) {
 		for (list = set->holders; list; list = list->next) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+							      G_CALLBACK (changed_holder_cb), set);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
 							      G_CALLBACK (validate_change_holder_cb), set);
-			if (! set->priv->read_only) {
-				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-								      G_CALLBACK (changed_holder_cb), set);
-				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-								      G_CALLBACK (source_changed_holder_cb), set);
-				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
-								      G_CALLBACK (att_holder_changed_cb), set);
-			}
-			g_object_unref (list->data);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+							      G_CALLBACK (source_changed_holder_cb), set);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
+							      G_CALLBACK (att_holder_changed_cb), set);
+			g_object_unref (G_OBJECT (list->data));
 		}
 		g_slist_free (set->holders);
 	}
@@ -1103,10 +1007,6 @@ gda_set_dispose (GObject *object)
 		g_hash_table_destroy (set->priv->holders_hash);
 		set->priv->holders_hash = NULL;
 	}
-	if (set->priv->holders_array) {
-		g_array_free (set->priv->holders_array, TRUE);
-		set->priv->holders_array = NULL;
-	}
 
 	/* free the nodes if there are some */
 	while (set->nodes_list)
@@ -1153,7 +1053,8 @@ compute_public_data (GdaSet *set)
 	GdaSetNode *node;
 	GdaSetSource *source;
 	GdaSetGroup *group;
-	GHashTable *groups = NULL;
+	GHashTable *groups = g_hash_table_new (NULL, NULL); /* key = source model, 
+							       value = GdaSetGroup */
 
 	/*
 	 * Get rid of all the previous structures
@@ -1178,14 +1079,14 @@ compute_public_data (GdaSet *set)
 		if (node->source_model)
 			g_object_ref (node->source_model);
 		
-		set->nodes_list = g_slist_prepend (set->nodes_list, node);
+		set->nodes_list = g_slist_append (set->nodes_list, node);
 	}
-	set->nodes_list = g_slist_reverse (set->nodes_list);
 
 	/*
 	 * Creation of the GdaSetSource and GdaSetGroup structures 
 	 */
-	for (list = set->nodes_list; list;list = list->next) {
+	list = set->nodes_list;
+	while (list) {
 		node = GDA_SET_NODE (list->data);
 		
 		/* source */
@@ -1193,18 +1094,18 @@ compute_public_data (GdaSet *set)
 		if (node->source_model) {
 			source = gda_set_get_source_for_model (set, node->source_model);
 			if (source) 
-				source->nodes = g_slist_append (source->nodes, node);
+				source->nodes = g_slist_prepend (source->nodes, node);
 			else {
 				source = g_new0 (GdaSetSource, 1);
 				source->data_model = node->source_model;
-				source->nodes = g_slist_append (NULL, node);
+				source->nodes = g_slist_prepend (NULL, node);
 				set->sources_list = g_slist_prepend (set->sources_list, source);
 			}
 		}
 
 		/* group */
 		group = NULL;
-		if (node->source_model && groups)
+		if (node->source_model)
 			group = g_hash_table_lookup (groups, node->source_model);
 		if (group) 
 			group->nodes = g_slist_append (group->nodes, node);
@@ -1212,18 +1113,15 @@ compute_public_data (GdaSet *set)
 			group = g_new0 (GdaSetGroup, 1);
 			group->nodes = g_slist_append (NULL, node);
 			group->nodes_source = source;
-			set->groups_list = g_slist_prepend (set->groups_list, group);
-			if (node->source_model) {
-				if (!groups)
-					groups = g_hash_table_new (NULL, NULL); /* key = source model, 
-										   value = GdaSetGroup */
+			set->groups_list = g_slist_append (set->groups_list, group);
+			if (node->source_model)
 				g_hash_table_insert (groups, node->source_model, group);
-			}
 		}		
+
+		list = g_slist_next (list);
 	}
-	set->groups_list = g_slist_reverse (set->groups_list);
-	if (groups)
-		g_hash_table_destroy (groups);
+	
+	g_hash_table_destroy (groups);
 
 #ifdef GDA_DEBUG_signal
         g_print (">> 'PUBLIC_DATA_CHANGED' from %p\n", set);
@@ -1261,43 +1159,15 @@ gda_set_add_holder (GdaSet *set, GdaHolder *holder)
 	return added;
 }
 
-static void
-holder_notify_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *dataset)
-{
-	GType gtype;
-	gtype = gda_holder_get_g_type (holder);
-	if (!strcmp (pspec->name, "g-type")) {
-		g_assert (gtype != GDA_TYPE_NULL);
-		g_signal_emit (dataset, gda_set_signals[HOLDER_TYPE_SET], 0, holder);
-	}
-	else if (!strcmp (pspec->name, "name")) {
-#ifdef GDA_DEBUG_signal
-	g_print (">> 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
-#endif
-	g_signal_emit (G_OBJECT (dataset), gda_set_signals[HOLDER_ATTR_CHANGED], 0, holder,
-		       GDA_ATTRIBUTE_NAME, gda_holder_get_attribute (holder, GDA_ATTRIBUTE_NAME));
-#ifdef GDA_DEBUG_signal
-	g_print ("<< 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
-#endif
-	}
-	else if (!strcmp (pspec->name, "description")) {
-#ifdef GDA_DEBUG_signal
-	g_print (">> 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
-#endif
-	g_signal_emit (G_OBJECT (dataset), gda_set_signals[HOLDER_ATTR_CHANGED], 0, holder,
-		       GDA_ATTRIBUTE_DESCRIPTION, gda_holder_get_attribute (holder, GDA_ATTRIBUTE_DESCRIPTION));
-#ifdef GDA_DEBUG_signal
-	g_print ("<< 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
-#endif
-	}
-}
-
 static gboolean
 gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 {
 	GdaHolder *similar;
 	const gchar *hid;
 
+	if (g_slist_find (set->holders, holder))
+		return FALSE;
+
 	/* 
 	 * try to find a similar holder in the set->holders:
 	 * a holder B is similar to a holder A if it has the same ID
@@ -1307,38 +1177,23 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 		g_warning (_("GdaHolder needs to have an ID"));
 		return FALSE;
 	}
-
+	g_return_val_if_fail (hid, FALSE);
 	similar = (GdaHolder*) g_hash_table_lookup (set->priv->holders_hash, hid);
 	if (!similar) {
 		/* really add @holder to the set */
 		set->holders = g_slist_append (set->holders, holder);
 		g_hash_table_insert (set->priv->holders_hash, (gchar*) hid, holder);
-		if (set->priv->holders_array) {
-			g_array_free (set->priv->holders_array, TRUE);
-			set->priv->holders_array = NULL;
-		}
 		g_object_ref (holder);
+		g_signal_connect (G_OBJECT (holder), "changed",
+				  G_CALLBACK (changed_holder_cb), set);
 		g_signal_connect (G_OBJECT (holder), "validate-change",
 				  G_CALLBACK (validate_change_holder_cb), set);
-		if (! set->priv->read_only) {
-			g_signal_connect (G_OBJECT (holder), "changed",
-					  G_CALLBACK (changed_holder_cb), set);
-			g_signal_connect (G_OBJECT (holder), "source-changed",
-					  G_CALLBACK (source_changed_holder_cb), set);
-			g_signal_connect (G_OBJECT (holder), "attribute-changed",
-					  G_CALLBACK (att_holder_changed_cb), set);
-		}
-		if (gda_holder_get_g_type (holder) == GDA_TYPE_NULL)
-			g_signal_connect (G_OBJECT (holder), "notify::g-type",
-					  G_CALLBACK (holder_notify_cb), set);
-		g_signal_connect (G_OBJECT (holder), "notify::name",
-				  G_CALLBACK (holder_notify_cb), set);
-		g_signal_connect (G_OBJECT (holder), "notify::description",
-				  G_CALLBACK (holder_notify_cb), set);
+		g_signal_connect (G_OBJECT (holder), "source-changed",
+				  G_CALLBACK (source_changed_holder_cb), set);
+		g_signal_connect (G_OBJECT (holder), "attribute-changed",
+				  G_CALLBACK (att_holder_changed_cb), set);
 		return TRUE;
 	}
-	else if (similar == holder)
-		return FALSE;
 	else {
 #ifdef GDA_DEBUG_NO
 		g_print ("In Set %p, Holder %p and %p are similar, keeping %p only\n", set, similar, holder, similar);
@@ -1396,7 +1251,7 @@ set_remove_source (GdaSet *set, GdaSetSource *source)
 /**
  * gda_set_is_valid:
  * @set: a #GdaSet object
- * @error: (allow-none): a place to store validation errors, or %NULL
+ * @error: a place to store validation errors, or %NULL
  *
  * This method tells if all @set's #GdaHolder objects are valid, and if
  * they represent a valid combination of values, as defined by rules
@@ -1463,36 +1318,6 @@ gda_set_get_holder (GdaSet *set, const gchar *holder_id)
 	return (GdaHolder *) g_hash_table_lookup (set->priv->holders_hash, holder_id);
 }
 
-/**
- * gda_set_get_nth_holder:
- * @set: a #GdaSet object
- * @pos: the position of the requested #GdaHolder, starting at %0
- *
- * Finds a #GdaHolder using its position
- *
- * Returns: (transfer none): the requested #GdaHolder or %NULL
- *
- * Since: 4.2
- */
-GdaHolder *
-gda_set_get_nth_holder (GdaSet *set, gint pos)
-{
-	g_return_val_if_fail (GDA_IS_SET (set), NULL);
-	g_return_val_if_fail (pos >= 0, NULL);
-
-	if (! set->priv->holders_array) {
-		GSList *list;
-		set->priv->holders_array = g_array_sized_new (FALSE, FALSE, sizeof (GdaHolder*),
-							      g_slist_length (set->holders));
-		for (list = set->holders; list; list = list->next)
-			g_array_append_val (set->priv->holders_array, list->data);
-	}
-	if ((guint)pos >= set->priv->holders_array->len)
-		return NULL;
-	else
-		return g_array_index (set->priv->holders_array, GdaHolder*, pos);
-}
-
 /**
  * gda_set_get_node:
  * @set: a #GdaSet object
@@ -1608,73 +1433,3 @@ gda_set_get_source_for_model (GdaSet *set, GdaDataModel *model)
 
 	return retval;
 }
-
-/**
- * gda_set_replace_source_model
- * @set: a #GdaSet object
- * @source: a pointer to a #GdaSetSource in @set
- * @model: a #GdaDataModel
- *
- * Replaces @source->data_model with @model, which must have the same
- * characteristics as @source->data_model (same column types)
- *
- * Also for each #GdaHolder for which @source->data_model is a source model,
- * this method calls gda_holder_set_source_model() with @model to replace
- * the source by the new model
- *
- * Since: 4.2
- */
-void
-gda_set_replace_source_model (GdaSet *set, GdaSetSource *source, GdaDataModel *model)
-{
-	g_return_if_fail (GDA_IS_SET (set));
-	g_return_if_fail (source);
-	g_return_if_fail (g_slist_find (set->sources_list, source));
-	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-	
-	/* compare models */
-	gint ncols, i;
-	ncols = gda_data_model_get_n_columns (source->data_model);
-	if (ncols != gda_data_model_get_n_columns (model)) {
-		g_warning (_("Replacing data model must have the same characteristics as the "
-			     "data model it replaces"));
-		return;
-	}
-	for (i = 0; i < ncols; i++) {
-		GdaColumn *c1, *c2;
-		GType t1, t2;
-		c1 = gda_data_model_describe_column (source->data_model, i);
-		c2 = gda_data_model_describe_column (model, i);
-		t1 = gda_column_get_g_type (c1);
-		t2 = gda_column_get_g_type (c2);
-
-		if ((t1 != GDA_TYPE_NULL) && (t2 != GDA_TYPE_NULL) && (t1 != t2)) {
-			g_warning (_("Replacing data model must have the same characteristics as the "
-				     "data model it replaces"));
-			return;
-		}
-	}
-
-	/* actually swap the models */
-	GSList *list;
-	source->data_model = model;
-	for (list = source->nodes; list; list = list->next) {
-		GdaSetNode *node = (GdaSetNode*) list->data;
-		g_object_unref (node->source_model);
-		node->source_model = g_object_ref (model);
-		g_signal_handlers_block_by_func (G_OBJECT (node->holder),
-						 G_CALLBACK (source_changed_holder_cb), set);
-		gda_holder_set_source_model (GDA_HOLDER (node->holder), model, node->source_column,
-					     NULL);
-		g_signal_handlers_unblock_by_func (G_OBJECT (node->holder),
-						   G_CALLBACK (source_changed_holder_cb), set);
-
-	}
-#ifdef GDA_DEBUG_signal
-	g_print (">> 'SOURCE_MODEL_CHANGED' from %s\n", __FUNCTION__);
-#endif
-	g_signal_emit (G_OBJECT (set), gda_set_signals[SOURCE_MODEL_CHANGED], 0, source);
-#ifdef GDA_DEBUG_signal
-	g_print ("<< 'SOURCE_MODEL_CHANGED' from %s\n", __FUNCTION__);
-#endif
-}
