@@ -1,6 +1,6 @@
 /* gda-statement.c
  *
- * Copyright (C) 2007 - 2008 Vivien Malerba
+ * Copyright (C) 2007 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -177,7 +177,7 @@ gda_statement_init (GdaStatement * stmt)
 }
 
 /**
- * gda_statement_new
+ * gda_statement_new:
  *
  * Creates a new #GdaStatement object
  *
@@ -194,7 +194,7 @@ gda_statement_new (void)
 
 
 /**
- * gda_statement_copy
+ * gda_statement_copy:
  * @orig: a #GdaStatement to make a copy of
  * 
  * Copy constructor
@@ -299,13 +299,13 @@ gda_statement_get_property (GObject *object,
 }
 
 /**
- * gda_statement_get_statement_type
+ * gda_statement_get_statement_type:
  * @stmt: a #GdaStatement object
  *
  * Get the type of statement held by @stmt. It returns GDA_SQL_STATEMENT_NONE if
  * @stmt does not hold any statement
  *
- * Returns: the statement type
+ * Returns: (transfer none): the statement type
  */
 GdaSqlStatementType
 gda_statement_get_statement_type (GdaStatement *stmt)
@@ -320,7 +320,7 @@ gda_statement_get_statement_type (GdaStatement *stmt)
 }
 
 /**
- * gda_statement_is_useless
+ * gda_statement_is_useless:
  * @stmt: a #GdaStatement object
  *
  * Tells if @stmt is composed only of spaces (that is it has no real SQL code), and is completely
@@ -364,7 +364,7 @@ gda_statement_is_useless (GdaStatement *stmt)
 }
 
 /**
- * gda_statement_check_structure
+ * gda_statement_check_structure:
  * @stmt: a #GdaStatement object
  * @error: a place to store errors, or %NULL
  * 
@@ -382,9 +382,9 @@ gda_statement_check_structure (GdaStatement *stmt, GError **error)
 }
 
 /**
- * gda_statement_check_validity
+ * gda_statement_check_validity:
  * @stmt: a #GdaStatement object
- * @cnc: a #GdaConnection object, or %NULL
+ * @cnc: (allow-none): a #GdaConnection object, or %NULL
  * @error: a place to store errors, or %NULL
  *
  * If @cnc is not %NULL then checks that every object (table, field, function) used in @stmt 
@@ -411,7 +411,7 @@ gda_statement_check_validity (GdaStatement *stmt, GdaConnection *cnc, GError **e
 }
 
 /**
- * gda_statement_normalize
+ * gda_statement_normalize:
  * @stmt: a #GdaStatement object
  * @cnc: a #GdaConnection object
  * @error: a place to store errors, or %NULL
@@ -432,7 +432,7 @@ gda_statement_normalize (GdaStatement *stmt, GdaConnection *cnc, GError **error)
 }
 
 /**
- * gda_statement_serialize
+ * gda_statement_serialize:
  * @stmt: a #GdaStatement object
  *
  * Creates a string representing the contents of @stmt.
@@ -496,9 +496,9 @@ get_params_foreach_func (GdaSqlAnyPart *node, GdaSet **params, GError **error)
 }
 
 /**
- * gda_statement_get_parameters
+ * gda_statement_get_parameters:
  * @stmt: a #GdaStatement object
- * @out_params: a place to store a new #GdaSet object, or %NULL
+ * @out_params: (out) (allow-none) (transfer full): a place to store a new #GdaSet object, or %NULL
  * @error: a place to store errors, or %NULL
  *
  * Get a new #GdaSet object which groups all the execution parameters
@@ -562,7 +562,7 @@ static gchar *default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderin
 static gchar *default_render_select_order (GdaSqlSelectOrder *order, GdaSqlRenderingContext *context, GError **error);
 
 /**
- * gda_statement_to_sql_real
+ * gda_statement_to_sql_real:
  * @stmt: a #GdaStatement object
  * @context: a #GdaSqlRenderingContext context
  * @error: a place to store errors, or %NULL
@@ -702,23 +702,25 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 }
 
 /**
- * gda_statement_to_sql_extended
+ * gda_statement_to_sql_extended:
  * @stmt: a #GdaStatement object
  * @cnc: a #GdaConnection object, or %NULL
  * @params: parameters contained in a single #GdaSet object
  * @flags: a set of flags to control the rendering
- * @params_used: a place to store the list of actual #GdaHolder objects in @params used to do the rendering, or %NULL
+ * @params_used: (element-type GdaHolder) (out) (transfer container) (allow-none):a place to store the list of actual #GdaHolder objects in @params used to do the rendering, or %NULL
  * @error: a place to store errors, or %NULL
  *
  * Renders @stmt as an SQL statement, with some control on how it is rendered.
  *
  * If @cnc is not %NULL, then the rendered SQL will better be suited to be used by @cnc (in particular
- * it may include some SQL tweaks and/or proprietary extensions specific to the database engine used by @cnc).
+ * it may include some SQL tweaks and/or proprietary extensions specific to the database engine used by @cnc):
+ * in this case the result is similar to calling gda_connection_statement_to_sql().
  *
  * Returns: a new string if no error occurred
  */
 gchar *
-gda_statement_to_sql_extended (GdaStatement *stmt, GdaConnection *cnc, GdaSet *params, GdaStatementSqlFlag flags, 
+gda_statement_to_sql_extended (GdaStatement *stmt, GdaConnection *cnc, GdaSet *params,
+			       GdaStatementSqlFlag flags, 
 			       GSList **params_used, GError **error)
 {
 	gchar *str;
@@ -731,6 +733,13 @@ gda_statement_to_sql_extended (GdaStatement *stmt, GdaConnection *cnc, GdaSet *p
 	context.params = params;
 	context.flags = flags;
 	if (cnc) {
+		GdaServerProvider *prov;
+		prov = gda_connection_get_provider (cnc);
+#define PROV_CLASS(provider) (GDA_SERVER_PROVIDER_CLASS (G_OBJECT_GET_CLASS (provider)))
+		if (prov && PROV_CLASS (prov)->statement_to_sql)
+			return (PROV_CLASS (prov)->statement_to_sql) (prov, 
+								      cnc, stmt, params, flags, 
+								      params_used, error);
 		context.cnc = cnc;
 		context.provider = gda_connection_get_provider (cnc);
 	}
@@ -784,6 +793,7 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 	GString *string;
 	gchar *str;
 	GSList *list;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (stmt, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_INSERT, NULL);
@@ -817,7 +827,10 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 
 	/* values */
 	if (stmt->select) {
-		g_string_append_c (string, ' ');
+		if (pretty)
+			g_string_append_c (string, '\n');
+		else
+			g_string_append_c (string, ' ');
 		str = context->render_select (GDA_SQL_ANY_PART (stmt->select), context, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -826,8 +839,12 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 	else {
 		for (list = stmt->values_list; list; list = list->next) {
 			GSList *rlist;
-			if (list == stmt->values_list)
-				g_string_append (string, " VALUES");
+			if (list == stmt->values_list) {
+				if (pretty)
+					g_string_append (string, "\nVALUES");
+				else
+					g_string_append (string, " VALUES");
+			}
 			else
 				g_string_append_c (string, ',');
 			for (rlist = (GSList*) list->data; rlist; rlist = rlist->next) {
@@ -837,6 +854,8 @@ default_render_insert (GdaSqlStatementInsert *stmt, GdaSqlRenderingContext *cont
 					g_string_append (string, ", ");
 				str = context->render_expr ((GdaSqlExpr*) rlist->data, context, NULL, NULL, error);
 				if (!str) goto err;
+				if (pretty && (rlist != (GSList*) list->data))
+					g_string_append (string, "\n\t");
 				g_string_append (string, str);
 				g_free (str);
 			}
@@ -895,6 +914,7 @@ default_render_update (GdaSqlStatementUpdate *stmt, GdaSqlRenderingContext *cont
 	GString *string;
 	gchar *str;
 	GSList *flist, *elist;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (stmt, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_UPDATE, NULL);
@@ -911,10 +931,18 @@ default_render_update (GdaSqlStatementUpdate *stmt, GdaSqlRenderingContext *cont
 	g_free (str);
 
 	/* columns set */
-	g_string_append (string, " SET ");
-	for (flist = stmt->fields_list, elist = stmt->expr_list; flist && elist; flist = flist->next, elist = elist->next) {
-		if (flist != stmt->fields_list)
+	if (pretty)
+		g_string_append (string, "\nSET ");
+	else
+		g_string_append (string, " SET ");
+	for (flist = stmt->fields_list, elist = stmt->expr_list;
+	     flist && elist;
+	     flist = flist->next, elist = elist->next) {
+		if (flist != stmt->fields_list) {
 			g_string_append (string, ", ");
+			if (pretty)
+				g_string_append (string, "\n\t");
+		}
 		str = context->render_field (GDA_SQL_ANY_PART (flist->data), context, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -928,7 +956,10 @@ default_render_update (GdaSqlStatementUpdate *stmt, GdaSqlRenderingContext *cont
 
 	/* cond */
 	if (stmt->cond) {
-		g_string_append (string, " WHERE ");
+		if (pretty)
+			g_string_append (string, "\nWHERE ");
+		else
+			g_string_append (string, " WHERE ");
 		str = context->render_expr (stmt->cond, context, NULL, NULL, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -1017,6 +1048,7 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 	GString *string;
 	gchar *str;
 	GSList *list;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (stmt, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_SELECT, NULL);
@@ -1033,6 +1065,8 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 			g_string_append_c (string, ' ');
 			g_free (str);
 		}
+		if (pretty)
+			g_string_append_c (string, '\n');
 	}
 	
 	/* selected expressions */
@@ -1041,6 +1075,8 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 		if (!str) goto err;
 		if (list != stmt->expr_list)
 			g_string_append (string, ", ");
+		if (pretty)
+			g_string_append (string, "\n\t");
 		g_string_append (string, str);
 		g_free (str);
 	}
@@ -1049,14 +1085,20 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 	if (stmt->from) {
 		str = context->render_select_from (GDA_SQL_ANY_PART (stmt->from), context, error);
 		if (!str) goto err;
-		g_string_append_c (string, ' ');
+		if (pretty)
+			g_string_append_c (string, '\n');
+		else
+			g_string_append_c (string, ' ');
 		g_string_append (string, str);
 		g_free (str);
 	}
 
 	/* WHERE */
 	if (stmt->where_cond) {
-		g_string_append (string, " WHERE ");
+		if (pretty)
+			g_string_append (string, "\nWHERE ");
+		else
+			g_string_append (string, " WHERE ");
 		str = context->render_expr (stmt->where_cond, context, NULL, NULL, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -1069,15 +1111,22 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 		if (!str) goto err;
 		if (list != stmt->group_by)
 			g_string_append (string, ", ");
-		else
-			g_string_append (string, " GROUP BY ");
+		else {
+			if (pretty)
+				g_string_append (string, "\nGROUP BY ");
+			else
+				g_string_append (string, " GROUP BY ");
+		}
 		g_string_append (string, str);
 		g_free (str);
 	}
 
 	/* HAVING */
 	if (stmt->having_cond) {
-		g_string_append (string, " HAVING ");
+		if (pretty)
+			g_string_append (string, "\nHAVING ");
+		else
+			g_string_append (string, " HAVING ");
 		str = context->render_expr (stmt->having_cond, context, NULL, NULL, error);
 		if (!str) goto err;
 		g_string_append (string, str);
@@ -1090,8 +1139,12 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 		if (!str) goto err;
 		if (list != stmt->order_by)
 			g_string_append (string, ", ");
-		else
-			g_string_append (string, " ORDER BY ");
+		else {
+			if (pretty)
+				g_string_append (string, "\nORDER BY ");
+			else
+				g_string_append (string, " ORDER BY ");
+		}
 		g_string_append (string, str);
 		g_free (str);
 	}
@@ -1832,6 +1885,7 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 {
 	GString *string;
 	gchar *str;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (join, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (join)->type == GDA_SQL_ANY_SQL_SELECT_JOIN, NULL);
@@ -1844,22 +1898,40 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 
 	switch (join->type) {
 	case GDA_SQL_SELECT_JOIN_CROSS:
-		string = g_string_new (",");
+		if (pretty)
+			string = g_string_new (",\n\t");
+		else
+			string = g_string_new (",");
 		break;
         case GDA_SQL_SELECT_JOIN_NATURAL:
-		string = g_string_new ("NATURAL JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tNATURAL JOIN");
+		else
+			string = g_string_new ("NATURAL JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_INNER:
-		string = g_string_new ("INNER JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tINNER JOIN");
+		else
+			string = g_string_new ("INNER JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_LEFT:
-		string = g_string_new ("LEFT JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tLEFT JOIN");
+		else
+			string = g_string_new ("LEFT JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_RIGHT:
-		string = g_string_new ("RIGHT JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tRIGHT JOIN");
+		else
+			string = g_string_new ("RIGHT JOIN");
 		break;
         case GDA_SQL_SELECT_JOIN_FULL:
-		string = g_string_new ("FULL JOIN");
+		if (pretty)
+			string = g_string_new ("\n\tFULL JOIN");
+		else
+			string = g_string_new ("FULL JOIN");
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1881,6 +1953,7 @@ default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *cont
 	}
 	str = context->render_select_target (GDA_SQL_ANY_PART (target), context, error);
 	if (!str) goto err;
+
 	g_string_append_c (string, ' ');
 	g_string_append (string, str);
 	g_free (str);
@@ -1934,6 +2007,7 @@ default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderingContext *cont
 	gchar *str;
 	GSList *tlist;
 	gint i;
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
 
 	g_return_val_if_fail (from, NULL);
 	g_return_val_if_fail (GDA_SQL_ANY_PART (from)->type == GDA_SQL_ANY_SQL_SELECT_FROM, NULL);
@@ -1950,14 +2024,20 @@ default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderingContext *cont
 		if (join) {
 			str = context->render_select_join (GDA_SQL_ANY_PART (join), context, error);
 			if (!str) goto err;
-			g_string_append_c (string, ' ');
+			if (!pretty)
+				g_string_append_c (string, ' ');
 			g_string_append (string, str);
 			g_free (str);
-			g_string_append_c (string, ' ');
+			if (!pretty)
+				g_string_append_c (string, ' ');
 		}
 		else {
-			if (tlist != from->targets)
-				g_string_append (string, ", ");
+			if (tlist != from->targets) {
+				if (pretty)
+					g_string_append (string, ",\n\t");
+				else
+					g_string_append (string, ", ");
+			}
 			str = context->render_select_target (GDA_SQL_ANY_PART (tlist->data), context, error);
 			if (!str) goto err;
 			g_string_append (string, str);
